/*
 *  Condition.hpp
 *  Copyright (C) 2001-2003 by Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

#ifndef Threading_Condition_hpp_
#define Threading_Condition_hpp_

#include "Threading/Mutex.hpp"

namespace Threading {

   /*! \brief %Condition for thread synchronization.
   \author Marc Parizeau

   This class incapsulates a cross-platform condition with classic Condition::broadcast, Condition::signal, and Condition::wait methods.
   */
   class Condition : public Mutex
   {
   public:
      Condition(void);
      ~Condition(void);

      void broadcast(void) const;
      void signal(void) const;
      void wait(void) const;
      
   protected:
      void* mCondition; //!< Opaque structure of native condition

   private:
      //! restrict (disable) copy constructor.
      Condition(const Condition&);
      //! restrict (disable) assignment operator.
      void operator=(const Condition&);
   };

} // end of Threading namespace

#endif // Threading_Condition_hpp_
