/*
 *  Exception.hpp
 *  Copyright (C) 2001-2003 by Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

#ifndef Threading_Exception_hpp_
#define Threading_Exception_hpp_

#include <stdexcept>
#include <string>

namespace Threading {

   //! Supported Multithread error codes.
   enum Error {
      eMutexNotOwned, //!< Mutex not own by calling thread
      eWouldDeadLock, //!< Operation would produce a dead lock.
      eOtherError //!< Any other OS specific error.
   };

   /*!
   \brief Multithread exception.
    \author Marc Parizeau

    This class encapsulates most interesting runtime errors that can happen while using multithreading operations.
    */
   class Exception : public std::runtime_error
   {
   public:
      //! Construct with native error code.
      explicit Exception(int inNativeCode, const std::string& inMessage)
      : runtime_error(inMessage), mNativeCode(inNativeCode) {mCode = convertNativeError(inNativeCode);}
      //! Construct with multithreading error code.
      explicit Exception(Error inCode, const std::string& inMessage) : runtime_error(inMessage), mCode(inCode) {mNativeCode = 0;}

      static Error convertNativeError(int inError);

      //! Return error code.
      int getErrorCode() {return mCode;}
      
      std::string getMessage() const throw();
      
   protected:
      Error mCode;
      int mNativeCode;
   
   };

} // end of Threading namespace

#endif  // Threading_Exception_hpp_
