/*
 *  Mutex.hpp
 *  Copyright (C) 2001-2003 by Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

#ifndef Threading_Mutex_hpp_
#define Threading_Mutex_hpp_

#include "Threading/Exception.hpp"

namespace Threading {

   /*! \brief Mutual exclusion for thread synchronization.
   \author Marc Parizeau

   This class incapsulates a mutex with classic Mutex::lock, Mutex::unlock, and Mutex::tryLock methods. A thread that already has acquired the mutex should not lock it again because this could lead to a deadlock. A thread should only unlock a mutex that is has lock previously. Otherwise the behavior of the mutex is undetermined (OS specific). 
   */
   class Mutex
   {
   public:
      Mutex(void);
      ~Mutex(void);
   
      void lock(void) const;
      bool tryLock(void) const;
      void unlock(void) const;
   
   protected:
      void* mMutex; //!< Opaque structure of native mutex

   private:
      //! restrict (disable) copy constructor.
      Mutex(const Mutex&);
      //! restrict (disable) assignment operator.
      void operator=(const Mutex&);
      
   };

} // end of Threading namespace

#endif // Threading_Mutex_hpp_
