/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GP/InitGrowConstrainedOp.hpp
 *  \brief  Definition of the class GP::InitGrowConstrainedOp.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.4 $
 *  $Date: 2004/07/04 16:58:22 $
 */

#ifndef Beagle_GP_InitGrowConstrainedOp_hpp
#define Beagle_GP_InitGrowConstrainedOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/InitializationOp.hpp"
#include "beagle/UInt.hpp"
#include "beagle/Bool.hpp"
#include "beagle/GP/Tree.hpp"
#include "beagle/GP/InitGrowOp.hpp"


namespace Beagle {
namespace GP {

/*!
 *  \class InitGrowConstrainedOp
 *    beagle/GP/InitGrowConstrainedOp.hpp "beagle/GP/InitGrowConstrainedOp.hpp"
 *  \brief Constrained GP trees "grow" initialization operator class.
 *  \ingroup GPF
 *  \ingroup GPOp
 *  \ingroup GPPop
 */
class InitGrowConstrainedOp : public InitGrowOp {

public:

  //! GP::InitGrowConstrainedOp allocator type.
  typedef AllocatorT<InitGrowConstrainedOp,InitGrowOp::Alloc>
          Alloc;
  //! GP::InitGrowConstrainedOp handle type.
  typedef PointerT<InitGrowConstrainedOp,InitGrowOp::Handle>
          Handle;
  //! GP::InitGrowConstrainedOp bag type.
  typedef ContainerT<InitGrowConstrainedOp,InitGrowOp::Bag>
          Bag;

  explicit InitGrowConstrainedOp(std::string inReproProbaName="ec.repro.prob",
                                 std::string inName="GP-InitGrowConstrainedOp");
  virtual ~InitGrowConstrainedOp() { }

  virtual void initialize(Beagle::System& ioSystem);
  virtual void initTreeGrow(GP::Tree& outTree,
                            unsigned int inMinDepth,
                            unsigned int inMaxDepth,
                            GP::Context& ioContext) const;

protected:
  unsigned int initConstrainedSubTreeGrow(GP::Tree& ioTree,
                                          unsigned int inMinDepth,
                                          unsigned int inMaxDepth,
                                          GP::Context& ioContext) const;

  UInt::Handle mNumberAttempts;   //!< Number of attempts in each initialization.
  
};

}
}

#endif // Beagle_GP_InitGrowConstrainedOp_hpp
