/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GP/Nor.hpp
 *  \brief  Definition of the templated class GP::Nor.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.3 $
 *  $Date: 2004/04/25 23:12:20 $
 */

#ifndef Beagle_GP_Nor_hpp
#define Beagle_GP_Nor_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/WrapperT.hpp"
#include "beagle/Bool.hpp"
#include "beagle/GP/Primitive.hpp"
#include "beagle/GP/PrimitiveInline.hpp"
#include "beagle/GP/Context.hpp"
#include "beagle/GP/Datum.hpp"


namespace Beagle {
namespace GP {

/*!
 *  \class Nor beagle/GP/Nor.hpp "beagle/GP/Nor.hpp"
 *  \brief GP boolean NOR primitive class.
 *  \ingroup GPF
 *  \ingroup Primit
 *  \ingroup BoolPrimit
 */
class Nor : public Primitive {

public:

  //! GP::Nor allocator type.
  typedef AllocatorT<Nor,Primitive::Alloc>
          Alloc;
  //! GP::Nor handle type.
  typedef PointerT<Nor,Primitive::Handle>
          Handle;
  //! GP::Nor bag type.
  typedef ContainerT<Nor,Primitive::Bag>
          Bag;

  explicit Nor(std::string inName="NOR");
  virtual ~Nor() { }

  virtual std::string getArgType(unsigned int inN) const;
  virtual std::string getReturnType() const;
  virtual void        execute(GP::Datum& outResult, GP::Context& ioContext);

};

}
}


#endif // Beagle_GP_Nor_hpp
