/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GP/PrimitiveSuperSet.hpp
 *  \brief  Definition of the type GP::PrimitiveSuperSet.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.3 $
 *  $Date: 2004/04/25 23:12:20 $
 */


#ifndef Beagle_GP_PrimitiveSuperSet_hpp
#define Beagle_GP_PrimitiveSuperSet_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/GP/PrimitiveSet.hpp"

namespace Beagle {
namespace GP {


// Forward declaration
class System;


/*!
 *  \class PrimitiveSuperSet beagle/GP/PrimitiveSuperSet.hpp "beagle/GP/PrimitiveSuperSet.hpp"
 *  \brief The GP primitive super set class.
 *  \ingroup GPF
 *  \ingroup Primit
 *  \ingroup GPSys
 */
class PrimitiveSuperSet : public PrimitiveSet::Bag {

public:

  //! GP::PrimitiveSuperSet allocator type.
  typedef AllocatorT<PrimitiveSuperSet,PrimitiveSet::Bag::Alloc>
          Alloc;
  //! GP::PrimitiveSuperSet handle type.
  typedef PointerT<PrimitiveSuperSet,PrimitiveSet::Bag::Handle>
          Handle;
  //! GP::PrimitiveSuperSet bag type.
  typedef ContainerT<PrimitiveSuperSet,PrimitiveSet::Bag::Bag>
          Bag;

           PrimitiveSuperSet() { }
  virtual ~PrimitiveSuperSet() { }

  virtual void initialize(GP::System& ioSystem);
  virtual void insert(PrimitiveSet::Handle inPrimitiveSet);
  virtual void read(XMLNode::Handle& inNode);
  virtual void write(XMLStreamer& ioStreamer) const;

};

}
}

#endif // Beagle_GP_PrimitiveSuperSet_hpp
