/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/IntegerVector.hpp
 *  \brief  Definition of the class IntegerVector.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.3 $
 *  $Date: 2004/04/25 23:12:20 $
 */

#ifndef Beagle_IntegerVector_hpp
#define Beagle_IntegerVector_hpp

#include <vector>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/Tokenizer.hpp"
#include "beagle/XMLNode.hpp"
#include "beagle/XMLParser.hpp"
#include "beagle/XMLStreamer.hpp"


namespace Beagle {

/*!
 *  \class IntegerVector beagle/IntegerVector.hpp "beagle/IntegerVector.hpp"
 *  \brief Unsigned integer vector class with a I/O format appropriate for a registred parameter.
 *  \ingroup OOF
 *  \ingroup Adapt
 */
class IntegerVector : public Object, public std::vector<unsigned int> {

public:

  //! IntegerVector allocator type.
  typedef AllocatorT<IntegerVector,Object::Alloc>
          Alloc;
  //! IntegerVector handle type.
  typedef PointerT<IntegerVector,Object::Handle>
          Handle;
  //! IntegerVector bag type.
  typedef ContainerT<IntegerVector,Object::Bag>
          Bag;

  explicit IntegerVector(unsigned int inSize=0, unsigned int inValue=0);
  virtual ~IntegerVector() { }

  virtual bool isEqual(const Object& inRightObj) const;
  virtual bool isLess(const Object& inRightObj) const;
  virtual void read(XMLNode::Handle& inNode);
  virtual void write(XMLStreamer& ioStreamer) const;

};

}

#endif // Beagle_IntegerVector_hpp
