/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/OperatorMap.hpp
 *  \brief  Definition of class OperatorMap.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.3 $
 *  $Date: 2004/04/25 23:12:21 $
 */

#ifndef Beagle_OperatorMap_hpp
#define Beagle_OperatorMap_hpp

#include <string>
#include <map>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Map.hpp"
#include "beagle/OperatorMap.hpp"


namespace Beagle {


/*!
 *  \class OperatorMap beagle/OperatorMap.hpp "beagle/OperatorMap.hpp"
 *  \brief Associative container using a std::string as key accessors to the
 *    Operator::Handle instances.
 *  \ingroup ECF
 *  \ingroup Op
 */
class OperatorMap : public Map {

public:

  //! OperatorMap allocator type.
  typedef AllocatorT<OperatorMap,Map::Alloc>
          Alloc;
  //! OperatorMap handle type.
  typedef PointerT<OperatorMap,Map::Handle>
          Handle;
  //! OperatorMap bag type.
  typedef ContainerT<OperatorMap,Map::Bag>
          Bag;

           OperatorMap() { }
  virtual ~OperatorMap() { }
  
  /*!
   *  \brief Get operator from the map.
   *  \param inKey Key associated to the operator.
   *  \return Smart pointer to the operator, a NULL pointer if the operator wasn't in the map.
   */
  inline Operator::Handle& operator[](std::string inKey)
  {
    return castHandleT<Operator>(Map::operator[](inKey));
  }

};

}

#endif // Beagle_OperatorMap_hpp


